/*
 * Created on Apr 10, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.imaginot.chess;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import com.imaginot.chess.swing.*;
import com.imaginot.chess.engine.*;
import chesspresso.position.*;
import chesspresso.move.*;
import chesspresso.*;
import com.imaginot.learning.*;

/**
 * @author dkappe
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class OpeningTest extends JPanel {
	private int totalQuestions = 0;
	private int totalRight = 0;
	private OpeningEntry current;
	private DefaultLearningBox lbox;
	private ChessBoard board;
	private ChessBoardModel model;
	private JTextField moveEntry = new JTextField(6);

	private JLabel status = new JLabel("");
	private JLabel score = new JLabel("");
	private JPanel subAssembly = new JPanel();
	private JPanel qPanel = new JPanel();
	private JPanel aPanel = new JPanel();
	private JComboBox validMoves = new JComboBox();
	
	private GuessAction guessAction = new GuessAction();
	private NextAction nextAction = new NextAction();

	public class GuessAction extends AbstractAction {
		public GuessAction() {
			super("Guess");
		}
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			checkResult();
			showScore();
			showAnswer();
		}

	}

	private void checkResult() {
		if (current.validMove(moveEntry.getText())) {
			totalRight++;
			status.setForeground(Color.BLACK);
			status.setText("Correct, " + moveEntry.getText().trim() +
				" is a book move.");
			lbox.answer(true); // it was a good one
		} else {
			status.setForeground(Color.RED);
			status.setText("Wrong, " + moveEntry.getText().trim() +
				" is not a book move.");
			lbox.answer(false);
		}
		
	}
	public class NextAction extends AbstractAction {

		public NextAction() {
			super("Next");
		}
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			nextPosition();
			showQuestion();
		}

	}

	private void showScore() {
		if (totalQuestions == 0) {
			score.setText("Score: N/A");
		} else {
			int percent = (100 * totalRight / totalQuestions);
			score.setText(
				"Score: "
					+ percent
					+ "% ("
					+ totalRight
					+ " of "
					+ totalQuestions
					+ ")");
		}
	}

	private void nextPosition() {
		Position p = null;
		try {
			//lbox.dump();
			current = (OpeningEntry)lbox.nextQuestion();
			p = new Position(current.getFen());
			setBoard(p);
		} catch (Exception e) {
			//lbox.dump();
			badBook();
			return;
		}
		status.setForeground(Color.BLACK);
		if (Position.isWhiteToPlay(p.getHashCode())) {
			status.setText("White to move. Give me a book move.");
		} else {
			status.setText("Black to move. Give me a book move.");
		}
		totalQuestions++;
	}

	public OpeningTest() {
		super();
		if (ChessVisualizationTrainer.book == null) { // just set a problem view for this page
			badBook();
			return;
		}
		// setup the learning box
		lbox = new DefaultLearningBox(new OpeningEntryGenerator(ChessVisualizationTrainer.book));
		
		// setup the board stuff
		model = new GenericChessBoardModel();
		board = new ChessBoard(model);
		
		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();

		gbc.insets = new Insets(5, 5, 5, 5);
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		
		status.setFont(new Font("San Serif", Font.PLAIN, 20));
		add(status,gbc);
		
		gbc.gridy = 1;
		add(board,gbc);
		
		
		
		gbc.gridy = 3;
		gbc.gridx = 0;
		add(score,gbc);
		
		gbc.gridy = 2;
		gbc.gridwidth = 3;
		gbc.weightx = 0.5;
		gbc.weighty = 0.5;
		subAssembly.setPreferredSize(new Dimension(400,55));
		subAssembly.setMinimumSize(qPanel.getPreferredSize());
		subAssembly.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		add(subAssembly, gbc);
		
		// build the two subassembly panels
		aPanel.setLayout(new GridLayout(1,3));
		aPanel.add(new JLabel("Book Moves:"));
		validMoves.setEditable(false);
		aPanel.add(validMoves);
		aPanel.add(new JButton(nextAction));
		//aPanel.setPreferredSize(new Dimension(400,50));
		//aPanel.setMaximumSize(aPanel.getPreferredSize());
		
		qPanel.setLayout(new GridLayout(1,2));
		qPanel.add(moveEntry);
		qPanel.add(new JButton(guessAction));
		//qPanel.setPreferredSize(new Dimension(400,100));
		//qPanel.setMinimumSize(qPanel.getPreferredSize());
		// set the question panel first
		subAssembly.add(qPanel);
		
		showScore();
		nextPosition();
		moveEntry.requestFocus();


	}

	private void setBoard(Position position) {

		char[][] retval = new char[8][8];
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				int piece = position.getStone((i * 8) + j);
				switch (piece) {
					case Chess.BLACK_BISHOP :
						retval[i][j] = 'b';
						break;
					case Chess.WHITE_BISHOP :
						retval[i][j] = 'B';
						break;
					case Chess.BLACK_KNIGHT :
						retval[i][j] = 'n';
						break;
					case Chess.WHITE_KNIGHT :
						retval[i][j] = 'N';
						break;
					case Chess.BLACK_ROOK :
						retval[i][j] = 'r';
						break;
					case Chess.WHITE_ROOK :
						retval[i][j] = 'R';
						break;
					case Chess.BLACK_QUEEN :
						retval[i][j] = 'q';
						break;
					case Chess.WHITE_QUEEN :
						retval[i][j] = 'Q';
						break;
					case Chess.BLACK_KING :
						retval[i][j] = 'k';
						break;
					case Chess.WHITE_KING :
						retval[i][j] = 'K';
						break;
					case Chess.BLACK_PAWN :
						retval[i][j] = 'p';
						break;
					case Chess.WHITE_PAWN :
						retval[i][j] = 'P';
						break;
					default :
						retval[i][j] = ' ';
				}
			}
		}
		model.updateBoard(retval);
	}
	
	private void showQuestion() {
		moveEntry.setText("");
		subAssembly.removeAll();
		subAssembly.add(qPanel);
		subAssembly.repaint();
		moveEntry.requestFocus();
	}
	
	private void showAnswer() {
		validMoves.removeAllItems();
		Iterator it = current.getMoveTexts().iterator();
		while (it.hasNext()) {
			String s = (String)it.next();
			//System.out.println("Move: '" + s + "'");
			validMoves.addItem(s);
		}
		subAssembly.removeAll();
		subAssembly.add(aPanel);
		subAssembly.repaint();
	}
	
	private void badBook() {
		removeAll();
		setLayout(new FlowLayout());
		JLabel l = new JLabel("Error with book.");
		l.setFont(new Font("Serif",Font.BOLD,24));
		add(l);
	}
}
