/*
 * Created on Apr 28, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.imaginot.chess;

import java.util.*;

import chesspresso.position.Position;
import chesspresso.move.Move;
import chesspresso.*;

/**
 * @author Administrator
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class PawnsAndMateInOneGenerator extends AbstractPositionGenerator {
	protected Position pawnPos;
	protected short lastWinner = Move.NO_MOVE;
	
	/**
	 * @param pieces
	 * @param side
	 */
	public PawnsAndMateInOneGenerator(
		List pieces,
		Position pawns,
		boolean side) {
		super(pieces, side);
		pawnPos = pawns;
	}

	/* (non-Javadoc)
	 * @see com.imaginot.chess.AbstractPositionGenerator#wantPosition()
	 */
	protected boolean wantPosition() {
		short winner = Move.ILLEGAL_MOVE;
		// first, the current mover cannot be in check and must be able to move
		if (pos.isCheck() || !pos.canMove()) {
			return false;
		}
		// if the black king is on the side of the board, that's too easy
		int bking = pos.getBlackKingSquare();
		int modulo = bking%8;
		if ((bking<8) || (bking>=56) || (modulo == 7) || (modulo == 0)) {
			return false;
		}
		// next, there must be one and only one move that makes this mate
		int mateCount = 0;
		short moves[] = pos.getAllMoves();
		for (int i = 0; i < moves.length; i++) {
			try {
				pos.doMove(moves[i]);
			} catch (Exception e) {
				return false; // error in move means we don't want this position
			}
			if (pos.isMate()) {
				winner = moves[i];
				mateCount++;
			}
			pos.undoMove();
			if (mateCount > 1) { // no unique mate
				return false;
			}
		}
		if (mateCount == 1) { // make sure it's not the same old position
			if (winner == lastWinner) {
				return false;
			} else {
				lastWinner = winner;
				return true;
			}
		}
		return false;
	}

	protected void clearBoard() {
		pos = new Position(pawnPos);
		pos.setToPlay(whiteToMove ? Chess.WHITE : Chess.BLACK);
	}
	
	public static void main(String args[]) {
		// get initial pawn position
		Position pawns = new Position();
		pawns.setStone(Chess.H3,Chess.WHITE_PAWN);
		pawns.setStone(Chess.G4,Chess.WHITE_PAWN);
		pawns.setStone(Chess.B2,Chess.WHITE_PAWN);
		pawns.setStone(Chess.F3,Chess.WHITE_PAWN);
		pawns.setStone(Chess.G5,Chess.BLACK_PAWN);
		pawns.setStone(Chess.F6,Chess.BLACK_PAWN);
		pawns.setStone(Chess.B3,Chess.BLACK_PAWN);
		pawns.setStone(Chess.F4,Chess.BLACK_KNIGHT);
		pawns.setStone(Chess.D5,Chess.BLACK_QUEEN);
		
		
		// now set up the piece list
		List l = new ArrayList();
		l.add(new Integer(Chess.WHITE_KING));
		l.add(new Integer(Chess.BLACK_KING));
		l.add(new Integer(Chess.WHITE_ROOK));
		l.add(new Integer(Chess.WHITE_KNIGHT));
		l.add(new Integer(Chess.WHITE_BISHOP));
		l.add(new Integer(Chess.BLACK_ROOK));
		l.add(new Integer(Chess.WHITE_QUEEN));
		
		// create the generator
		PawnsAndMateInOneGenerator mg = new PawnsAndMateInOneGenerator(
		l, pawns, true); // white to move
		
		for (int i = 0; i < 20; i++) {
			Position p = mg.genPosition();
			System.out.println(PositionDescriber.describe(p));
			System.out.println(showForcedMate(p));
			System.out.println("-----------------------");
		}
		System.out.println("Done");
	}
	
	public static String showForcedMate(Position p) {
			// first, the current mover cannot be in check and must be able to move
			if (!p.canMove()) {
				return "No move.\n";
			}
			// next, gather up the moves that make this mate
			
			StringBuffer buff = new StringBuffer("Moves:");
			
			short moves[] = p.getAllMoves();
			for (int i = 0; i < moves.length; i++) {
				try {
					p.doMove(moves[i]);
				} catch (Exception e) {
					return buff.toString(); // error in move means we don't want this position
				}
				if (p.isMate()) {
					buff.append(" " + Move.getString(moves[i]));
				}
				p.undoMove();
			}
			return buff.toString();
		}
}
